function [u]=CVMST_Seg(Img,lambda,mu,sigma)
%--------------------------------------------------------------------------
%   This program is using the Split-Bregman method to solve a convex variant
%   of the Mumford-Shah model proposed in our paper [1].
%    
%   Usage: [uu,te,er]=CVMST_Seg(Img,lambda,mu,sigma);
%
%   Inputs: 
%       - Img: the given image
%       - lambda: the fidelity parameter
%       - mu: the smooth term parameter
%       - sigma: the Split-Bregman parameter
%
%   Outputs: 
%       - u: the solution of our convex variant of the Mumford-Shah model
%
%
%   The original algorithm was proposed in the following paper, if you use 
%   this file or package for your work, please refer to the following paper:
% 
%   [1] Xiaohao Cai, Raymond H. Chan, and Tieyong Zeng,
%       A Two-Stage Image Segmentation Method Using a Convex Variant 
%       of the Mumford-Shah Model and Thresholding, accepted for 
%       publication by SIAM Journal on Imaging Sciences.
%
%   Code by: Xiaohao Cai
%   Last updated: 10/11/2012 
%--------------------------------------------------------------------------




%%
% initialization
u=Img;wx=0*Img;wy=wx;bx=wx;by=wx;

% Forward difference matrices
[xLen,yLen]=size(Img);
[DDx,DDy] = first_order_differences(xLen,yLen);

% Build Kernels: use the fft algorithm: (5 point stencil)
uker = 0*Img;
uker(1,1) = 4;uker(1,2)=-1;uker(2,1)=-1;
uker(end,1)=-1;uker(1,end)=-1;  

% compute the fft of the left hand side of (3.6) in our paper
uker = lambda+(mu+sigma)*fft2(uker);

%%
for i=1:300
    % compute the right hand side of (3.6) in our paper
    rhs = lambda.*Img+sigma*DDx'*(wx-bx)+sigma*(wy-by)*DDy;
        
    u0=u;
    
    % solve (3.6) in our paper
    u = ifft2(fft2(rhs)./uker);
    err=norm(u-u0,'fro')/norm(u,'fro');
    
    if mod(i,10)==0
        disp(['iterations: ' num2str(i) '!  ' 'error is:   ' num2str(err)]);
    end
    
    % check the stopping criterion
    if err<10^(-4)
        break;
    end
    
    % solve (3.7) in our paper
    temp1=DDx*u+bx;temp2=u*DDy'+by; 
    [wx,wy]=shrink2(temp1,temp2,1./sigma);
    
    % update (3.3) in our paper
    bx=bx+DDx*u-wx;
    by=by+u*DDy'-wy;  
end

disp(['All iterations: ' num2str(i)]);

% linear streach, i.e. (3.8) in our paper
min_u=min(min(u));
max_u=max(max(u));
u=(u-min_u)/(max_u-min_u);

%--------------------------------------------------------------------------
function [H1_l,H1_r] = first_order_differences(m,n)
%--------------------------------------------------------------------------
%derivatives at the right and lower boundary are set to zero (mirroring)

d1 = [-ones(m-1,1); 0];
d2 = ones(m,1);
H1_l = spdiags([d1,d2],[0,1],m,m);

d1 = [-ones(n-1,1); 0];
d2 = ones(n,1);
H1_r = spdiags([d1,d2],[0,1],n,n);



