% function CVMST_SegMainNoise
%--------------------------------------------------------------------------
%   A Three-stage Approach for Segmenting Degraded Color Images: 
%       Smoothing, Lifting and Thresholding (SLaT)
%
%   Usage: CVMST_SegMainNoise;
% 
%   The original algorithm was proposed in the following paper, if you use 
%   this file or package for your work, please refer to the following
%   paper:
% 
%   [1] X. Cai, R. Chan, M. Nikolova, and T. Zeng. 
%       A Three-stage Approach for Segmenting Degraded Color Images: 
%       Smoothing, Lifting and Thresholding (SLaT), submitted. 
% 
%   [2] X. Cai, R. Chan, and T. Zeng,
%       A Two-Stage Image Segmentation Method Using a Convex Variant 
%       of the Mumford-Shah Model and Thresholding, accepted for 
%       publication by SIAM Journal on Imaging Sciences.
%  
% 
%   parameters:
%       - lambda: the fidelity parameter
%       - mu: the smooth term parameter
%       - sigma: the Split-Bregman parameter
%       - k: the number of phases 
% 
% 
%   All Rights Reserved  
%   Code by: Xiaohao Cai
%   Last updated: 25/08/2016
%--------------------------------------------------------------------------

close all; 
clear all; clc;

addpath('./data');

%% load data (flower.jpg) and set parameters
f=double(imread('flowers.jpg'))/255;
Img_clean=f;
load flower_noise;
lambda=3;mu=1;sigma=2;k=4;


%% our Two-Stage Image Segmentation Method
tic;
%-- Stage-one: solve our convex variant of the Mumford-Shah model
uu=zeros(size(Img));
zLen=size(uu,3);
for i=1:zLen
    uu(:,:,i)=CVMST_Seg(Img(:,:,i),lambda,mu,sigma);
end

%-- Stage-two: lifting 
cform = makecform('srgb2lab');
g = double(applycform(uint8(uu*255),cform))/255; 
uu(:,:,4:6)=g;
% scale to [0, 1]
for i=4:6
    temp=uu(:,:,i);
    max_t=max(temp(:)); min_t=min(temp(:));
    if max_t==min_t
        uu(:,:,i) = temp.*0;
    else
        uu(:,:,i)=(temp-min_t)/(max_t-min_t);
    end
end

%-- Stage-three: choose k mean values automatically using K-Means method
mean_u=MeanKmeans(uu,k);
% sort vector images
mean_u_s=SortMultiVec(mean_u);
% compute seg 
[flag, color]=colordisplay2(uu,mean_u_s,Img_clean);
toc;

%% show results
figure, imshow(Img(:,:,1:3),'border','tight');    % noisy image
set(gca,'position',[0 0 1 1],'units','normalized')
figure, imshow(uu(:,:,1:3),'border','tight');     % smooth image
set(gca,'position',[0 0 1 1],'units','normalized')
figure, imshow(color,'border','tight');           % segmentation result
set(gca,'position',[0 0 1 1],'units','normalized')




